/*--------------------------------------------------------------------

  World Folio - Offline map viewer/editor for Age of Adventure

  Copyright (c) 2001 Phil Dempster <phil@buxton.port5.com>
  http://buxton.port5.com/age/

  This software is based on an original work the copyright in which
  vests in Interactive Broadcasting Ltd (http://www.ibgames.net/).
  Original programming by Alan Lenton with additional programming by
  Stanley Lenton.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Compiled with Borland C++ Builder 3.

--------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "age_map.h"

#include <cstdio>

using namespace std;

#include "location.h"
#include "main.h"
#include "index.h"
#include "notes.h"
#include "pngdib.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


//Graphics::TBitmap   *AgeMap::map_gfx = 0;

bool AgeMap::zoom = false;

static LPBITMAPINFOHEADER lpdib;
static void *lpbits;

static int parse_loc(char *p, FILE *is)
{
    static char buf[2048];
    int locn, exits;

    sscanf(p, "%d,%d,", &locn, &exits); // Extract col, row
    sprintf(p, "%d:%d:%d:", locn % 32, locn / 32, exits);

    fgets(buf, sizeof(buf), is);        // Drop this line.
    fgets(buf, sizeof(buf), is);        // Loc name.
    strcat(p, buf);
    p[strlen(p) - 1] = ':';             // Replace \n with :

    fgets(buf, sizeof(buf), is);        // Loc descr.
    strcat(p, buf);                     // Don't drop \n

    fgets(buf, sizeof(buf), is);        // Check end line.
    return (strcmp(buf, "---end item---\n")==0) ? 1 : 0;
}

/*----------------------------------
    Construct a map from a given file
    Params: File stream pointer
----------------------------------*/
AgeMap::AgeMap(FILE *file)
{
    Location    *loc;
    int errors = 0;
    char    *buffer = new char[2048];

    loc_list.erase(loc_list.begin(),loc_list.end());

    if(fgets(title,80,file) == 0)
        MessageDlg("Empty file!",mtInformation,TMsgDlgButtons() << mbOK,0);
    else
    {
        title[strlen(title) - 1] = '\0';        // remove the '\n'

        filetype = (strcmp(title, "AoA Town file version 2")==0) ? DESIGN : MAP;

        while(fgets(buffer,2048,file) != 0)
        {
            if (filetype == DESIGN)
                parse_loc(buffer, file);

            loc = new Location(buffer);
            if(!loc->IsOK())
            {
                errors++;
                delete loc;
            }
            else
                loc_list[loc->Column() + 32 * loc->Row()] = loc;
        }
    }

    if(errors > 0)
    {
        sprintf(buffer,"%d locations had errors",errors);
        MessageDlg(buffer,mtInformation,TMsgDlgButtons() << mbOK,0);
    }
    else
    {
        sprintf(buffer,"%d locations loaded",loc_list.size());
        MainForm->Status->Panels->Items[2]->Text = buffer;
    }

    sprintf(buffer,"Looking at map of %s",title);
    MainForm->Caption = buffer;

    delete [] buffer;

    return;
}

AgeMap::~AgeMap()
{
    Clear();
}



/*-----------------------------------
    AgeMap::CheckDiagonalLink()
    Find out if there is an left/right
    location link in this cell.
    Params: Grid col & row
                Tile size in pixels
    Returns:    Source rect left coord
-----------------------------------*/
int AgeMap::CheckDiagonalLink(int GridCol,int GridRow,int tile_size)
{
    int diag = 0;

    if(CheckExit(GridCol - 1,GridRow + 1,2))    // sw/ne link?
        diag = 5 * tile_size;

    if(CheckExit(GridCol + 1,GridRow - 1,32))
        diag = 5 * tile_size;

    if(CheckExit(GridCol - 1,GridRow - 1,8) || CheckExit(GridCol + 1,GridRow + 1,128))  // se/nw link?
    {
        if(diag == 0)
            diag = 6 * tile_size;
        else
            diag = 7 * tile_size;       // crossed diagonals!!
    }

    return(diag);
}

/*----------------------------------
    AgeMap::CheckExit()
    Find out if there is an exit
    in the specified direction
    Params: The row and col coords
                The direction
    Returns:    TRUE if there is an exit
                else FALSE
----------------------------------*/
bool    AgeMap::CheckExit(int grid_col,int grid_row, int dir)
{
    LocList::iterator   iter;
    int exits;

    if((grid_col > 0) && (grid_row > 0))
    {
        int cur_col = grid_col / 2 + Location::mincol;
        int cur_row = grid_row / 2 + Location::minrow;
        int loc_no  = cur_row * 32 + cur_col;

        if((loc_no >= 0) && (loc_no < 32 * 32))
        {
            if((iter = loc_list.find(loc_no)) == loc_list.end())        // Not a real location
                return(false);
            exits = ((*iter).second)->Exits();
            if((exits & dir) == dir)
                return(true);
        }
    }

    return(FALSE);
}

/*----------------------------------------------------
    AgeMap::CheckLeftRightLink()
    Find out if there is an left/rt location link in
    this cell.
    Params: Grid col & row
                Tile size in pixels
    Returns:    Source rect left coord
----------------------------------------------------*/
int AgeMap::CheckLeftRightLink(int GridCol,int GridRow,int tile_size)
{
    if(CheckExit(GridCol - 1,GridRow,4))
        return(4 * tile_size);
    if(CheckExit(GridCol + 1,GridRow,64))
        return(4 * tile_size);

    return(0);
}

/*--------------------------------------------------
    AgeMap::CheckUpDownLink()
    Find out if there is an up/down location link in
    this cell.
    Params: Grid col & row
                Tile size in pixels
    Returns:    Source rect left coord
--------------------------------------------------*/
int AgeMap::CheckUpDownLink(int GridCol,int GridRow,int tile_size)
{
    if(CheckExit(GridCol,GridRow - 1,16))
        return(3 * tile_size);
    if(CheckExit(GridCol,GridRow + 1,1))
        return(3 * tile_size);

    return(0);
}

/*----------------------------------
    AgeMap::Clear()
    Delete all the existing locations
    on the map and delete the list.
----------------------------------*/
void    AgeMap::Clear()
{
    Location    *loc;
    while(loc_list.size() > 0)
    {
        loc = (*(loc_list.begin())).second;
        loc_list.erase(loc_list.begin());
        delete loc;
    }
}

/*------------------------------------------
    AgeMap::DrawCell()
    Draw he specified cell on the map display
    Params: The column of the grid
                The row of the gris
                The rectangle to draw on
                Tile size in pixels
------------------------------------------*/
void    AgeMap::DrawCell(int GridCol,int GridRow,TRect &rect,int tile_size)
{
    //TRect   source;
    int x, y;
    LocList::iterator   iter;
    int loc_num, special;

    x = 0;                                        // Default to blank

    // Figure out what we should draw in this cell...
    if((GridCol >= 0) && (GridRow >= 0))                    // should always be the case, but...
    {
        if(IsPossibleLoc(GridCol,GridRow))
        {
            loc_num = (GridRow/2 + Location::minrow) * 32
                    + (GridCol/2 + Location::mincol);

            if((iter = loc_list.find(loc_num)) != loc_list.end())   // Location exists
            {
                if ((special = ((*iter).second)->Special()) > I_DEFAULT)
                {                                           // Its special
                    x = special * tile_size;
                }
                else if (loc_num == IndexForm->FindLoc())   // Search result
                {
                    x = I_CROSS * tile_size;
                }
                else if (((*iter).second)->Notes() != 0)    // Has notes
                {
                    x = I_NOTES * tile_size;
                }
                else                                        // Normal location
                {
                    x = I_DEFAULT * tile_size;
                }
            }
        }
        else
        {
            if((GridCol % 2) == 1)
                x = CheckUpDownLink(GridCol,GridRow,tile_size);
            if((GridRow % 2) == 1)
                x = CheckLeftRightLink(GridCol,GridRow,tile_size);
            if(((GridRow % 2) == 0) && ((GridCol % 2) == 0))
                x = CheckDiagonalLink(GridCol,GridRow,tile_size);
        }
    }

    // ...and then draw it!

    //(tile_size*3)/2 : 0;
    //MainForm->Grid->Canvas->CopyRect(rect, map_gfx->Canvas, rect);

    y = Zoom() ? 0 : 16;

    HDC hdc = MainForm->Grid->Canvas->Handle;
    int prevstretchmode=SetStretchBltMode(hdc, COLORONCOLOR);
    StretchDIBits(hdc,
        rect.Left,   rect.Top, rect.Right-rect.Left, rect.Bottom-rect.Top,   /* dest  */
        x, y, tile_size, tile_size,   /* source  */
        lpbits, (LPBITMAPINFO)lpdib,
        DIB_RGB_COLORS, SRCCOPY);

    if (prevstretchmode)
    {
        SetStretchBltMode(hdc, prevstretchmode);
    }
}

/*-----------------------------
    AgeMap::IsPossibleLoc()
    Check to see if this cell
    holds locations or links
    Params: The row and column
    Returns:    TRUE if its a loc
                FALSE if its a link
-----------------------------*/
bool    AgeMap::IsPossibleLoc(int grid_col,int grid_row)
{
    if(((grid_col % 2) == 1) && ((grid_row % 2) == 1))
        return(TRUE);
    else
        return(FALSE);
}

/*--------------------------
    AgeMap::LoadMapGfx()
    Load in the graphics used
    by the map
--------------------------*/
void    AgeMap::LoadMapGfx()
{
    //map_gfx = new Graphics::TBitmap;

    PNGD_P2DINFO p2d;

    /* Read from file */
    ZeroMemory((void*)&p2d, sizeof(PNGD_P2DINFO));

    p2d.structsize=sizeof(PNGD_P2DINFO);
    p2d.pngfn=".\\grafix\\newlink.png";

    if (read_png_to_dib(&p2d)!=0)  /* Returns 0 on success. */
    {
        MessageDlg("Can't read newlink.png", mtInformation, TMsgDlgButtons() << mbOK, 0);
        return;
    }

    /* get rid of previous image */
    DeleteMapGfx();

    lpdib  = p2d.lpdib;
    lpbits = p2d.bits;
}

void AgeMap::DeleteMapGfx(void)
{
    if (lpdib)
    {
        GlobalFree(lpdib);
        lpdib=NULL;
    }

}

/*--------------------------------------------
    AgeMap::Name2Status()
    Write the name of the location the cursor
    is over into the status bar. If the cursor
    isn't over a location, clear the status bar
    Params: The grid cell coordinates
--------------------------------------------*/
void    AgeMap::Name2Status(int grid_col,int grid_row)
{
    char buffer[160];

    if(IsPossibleLoc(grid_col, grid_row))
    {
        int cur_col = grid_col / 2 + Location::mincol;
        int cur_row = grid_row / 2 + Location::minrow;
        int loc_no  = cur_row * 32 + cur_col;

        LocList::iterator iter = loc_list.find(loc_no);
        if(iter != loc_list.end())      // yes - it's a location
        {
            sprintf(buffer, "%d",(*iter).first);
            MainForm->Status->Panels->Items[0]->Text = buffer;

            sprintf(buffer, "%d, %d", cur_col, cur_row);
            MainForm->Status->Panels->Items[1]->Text = buffer;

            MainForm->Status->Panels->Items[2]->Text = ((*iter).second)->Name();
            return;
        }
    }
    MainForm->Status->Panels->Items[0]->Text = "";
    MainForm->Status->Panels->Items[1]->Text = "";
    MainForm->Status->Panels->Items[2]->Text = "";

    return;
}

/*-----------------------------
    Agemap::Save()
    Save this map out to disk
    Params: File stream pointer
-----------------------------*/
void    AgeMap::Save(FILE *file)
{
    fprintf(file,"%s\n",title);

    for(LocList::iterator iter = loc_list.begin();iter != loc_list.end();iter++)
        ((*iter).second)->Save(file);

    return;
}

/*-----------------------------------
    AgeMap::ShowNotes()
    Show the notes for this location
    Params: The grid cell coordinates
-----------------------------------*/
void    AgeMap::ShowNotes(int grid_col,int grid_row,int X, int Y)
{
    if(IsPossibleLoc(grid_col, grid_row))
    {
        int cur_col = grid_col / 2 + Location::mincol;
        int cur_row = grid_row / 2 + Location::minrow;
        int loc_no  = cur_row * 32 + cur_col;

        LocList::iterator iter = loc_list.find(loc_no);
        if(iter != loc_list.end())      // yes - it's a location
            ((*iter).second)->ShowNotes();
    }

    return;
}


/*-----------------------------------
    AgeMap::FindNames()
    Fill FindBox with all locations in database.
    Params: None.
-----------------------------------*/
void AgeMap::FindNames(void)
{
    IndexForm->IndexList->Enabled = false;
    IndexForm->IndexList->Clear();

    // 1. Iterate thru' Location objects held in loc_list container.
    for(LocList::iterator iter = loc_list.begin();
        iter != loc_list.end();
        iter++)
    {
    // 2. Add the name field to FindBox.
        IndexForm->IndexList->Items->AddObject(((*iter).second)->Name(), (TObject*)((*iter).second));
    }
    IndexForm->IndexList->Enabled = true;
}
