/*--------------------------------------------------------------------

  World Folio - Offline map viewer/editor for Age of Adventure
  
  Copyright (c) 2001 Phil Dempster <phil@buxton.port5.com>
  http://buxton.port5.com/age/

  This software is based on an original work the copyright in which
  vests in Interactive Broadcasting Ltd (http://www.ibgames.net/).
  Original programming by Alan Lenton with additional programming by
  Stanley Lenton.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Compiled with Borland C++ Builder 3.
    
--------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "location.h"

#include <cstring>
#include <stdlib>

using namespace std;

#include "main.h"
#include "notes.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

int Location::mincol = 64;
int Location::maxcol = -1;
int Location::minrow = 64;
int Location::maxrow = -1;

/*-------------------------------------
    Construct a location from a line of
    text with colon delimited fields.
    The order is: column:row:exits:name:
    Params: Pointer to the text
-------------------------------------*/
Location::Location(char *text)
{
    char *token;
    
    special = user = -1;
    notes = 0;

    if((token = strtok(text,":")) == 0)
        column = row = -1;
    else
    {
        column = atoi(token);
        if((token = strtok(0,":")) == 0)
            column = row = -1;
        else
        {
            row = atoi(token);
            if((token = strtok(0,":")) == 0)
                column = row = -1;
            else
            {
                exits =  atoi(token);
                if((token = strtok(0,":\n")) == 0)
                    column = row = -1;
                else
                {
                    // Set up class (static) vars to use in centering map.
                    if (column < mincol)
                        mincol = column;
                    else if (column > maxcol)
                        maxcol = column;
                    if (row < minrow)
                        minrow = row;
                    else if (row > maxrow)
                        maxrow = row;
                    
                    strncpy(name,token,MAX_NAME);
                    name[MAX_NAME - 1] =  '\0';

                    // Pre-process name for special tags.
                    special = CheckAutomagically(name);

                    if ((token = strtok(0, ":\n")) != 0)
                    {
                        // Pre-process notes for user tags.
                        user = CheckSpecialLoc(token);
                        
                        if (user == -1 || ((token = strtok(0, ":\n")) != 0))
                        {
                            //strtok(0, "\n");            // Lose the newline.
                            notes = new char[1280];     // cut a bit of slack :)
                            strcpy(notes, token);
                        }
                    }
                }
            }
        }
    }
}

int Location::CheckAutomagically(const char *p)
{
    // Only check words i.e. first character and characters following a space.
    bool in_word = true;

    while (*p)
    {
        if (*p == ' ')
        {
            p++;
            in_word = true;
        }
        
        if (!in_word)
        {
            p++;
            continue;
        }
        
        switch (*p++)
        {
        case 'A':
            if (strncmp(p, "irship", 6)==0)
                return I_AIRSHIP;
            break;
        case 'B':
            if (strncmp(p, "ahnhof", 6)==0)
                return I_RAILWAY;
            break;
        case 'C':
            if (strncmp(p, "lub", 3)==0)
                return I_INSTITUTE;
            break;
        case 'G':
            if (strncmp(p, "are", 3)==0)
                return I_RAILWAY;
            break;
        case 'H':
            if (strncmp(p, "ospital", 7)==0)
                return I_HOSPITAL;
            break;
        case 'S':
        case 's':   // For Buffalo, USA :-(
            // Check its not part of 'Stationary'.
            if (strncmp(p, "tation", 6)==0 && (*(p+6)=='\0' || *(p+6)==' '))
                return I_RAILWAY;
            break;
        case 'T':
            if (strncmp(p, "elegraph", 8)==0)
                return I_TELEGRAPH;
            break;
        default:
            break;
        }
        in_word = false;
    }
    return -1;
}

int Location::CheckSpecialLoc(const char *p)
{
    if (*p++ == '@')
    {
        switch (*p++)
        {
        case 'C':
            if (strncmp(p, "AFEBAR", 6)==0)
                return I_CAFEBAR;
            break;
        case 'H':   // Backwards compatibility.
            if (strncmp(p, "ARBOUR", 6)==0)
                return I_PORT;
            break;
        case 'P':
            if (strncmp(p, "ORT", 3)==0)
                return I_PORT;
            break;
        case 'R':
            // For the odd ones that aren't picked up (eg Baghdad, Damascus).
            if (strncmp(p, "AILWAY", 6)==0)
               return I_RAILWAY;

            if (strncmp(p, "ATION", 5)==0)
                return I_RATIONS;
            break;
        case 'S':
            if (strncmp(p, "KILL", 4)==0)
                return I_SKILL;

            // Backwards compatibility.
            if (strncmp(p, "TATION", 6)==0)
                return I_PORT;
            break;
        case 'W':
            if (strncmp(p, "EAPON", 5)==0)
                return I_WEAPON;
            break;
        default:
            break;
        }
        return I_DEFAULT;   // For overiding special locs.
    }
    return -1;   
}                       


Location::~Location()
{
    if (notes)
        delete [] notes;
}


/*----------------------------
    Location::Save()
    Save this location to file
    Params: Ptr to file stream
----------------------------*/
void    Location::Save(FILE *file)
{
    fprintf(file,"%d:%d:%d:%s",column,row,exits,name);
    
    switch (Special())
    {
//        case I_PAWNSHOP:fprintf(file, ":%s", "@PAWNSHOP");  break;
        case I_CAFEBAR: fprintf(file, ":%s", "@CAFEBAR");   break;
        case I_RAILWAY: fprintf(file, ":%s", "@RAILWAY");   break;
        case I_PORT:    fprintf(file, ":%s", "@PORT");      break;
        case I_RATIONS: fprintf(file, ":%s", "@RATIONS");   break;
        case I_WEAPON:  fprintf(file, ":%s", "@WEAPON");    break;
        case I_SKILL:   fprintf(file, ":%s", "@SKILL");     break;
        case I_DEFAULT: fprintf(file, ":%s", "@");          break;
        default:        break;
    }

    if (notes == 0)
        fprintf(file, "\n");
    else
        fprintf(file, ":%s\n",notes);

    return;
}

/*---------------------------------
    Location::ShowNotes()
    Show the notes for this location
  Params: None
---------------------------------*/
void    Location::ShowNotes(void)
{
    char   *token,temp[1280];
    String str2="";

    if(!NotesForm->IsClean())
    {
        switch(MessageDlg("You have made changes -\ndiscard?",mtConfirmation,TMsgDlgButtons() << mbOK << mbCancel,0))
        {
        case mrOk:
            break;
        case mrCancel:
            return;
        }
    }

    NotesForm->SetLoc(this);
    NotesForm->Caption = name;
    NotesForm->Status();
    NotesForm->NotesMemo->Clear();

    if (notes != 0)
    {
        strcpy(temp, notes);
        token = strtok(temp, "~\n\r");
        while(token != 0)
        {
            NotesForm->NotesMemo->Lines->Add(token);
            str2 += token;                  // put notes into str2
            token = strtok(0, "~\n");
        }
    }
    NotesForm->Show();
    NotesForm->IsClean(true);

    return;
}

/*-----------------------------------
    Location::UpdateNotes()
    Update the notes for this location
    from the entry on the notes form
-----------------------------------*/
void    Location::UpdateNotes()
{
    if(notes == 0)
        notes = new char[1280];

    strcpy(notes, NotesForm->NotesMemo->Text.c_str());
    if(notes[0] == '\0')
    {
        delete [] notes;
        notes = 0;
    }
    else
    {
        for(int count = 0;notes[count] != '\0';count++)
        {
            if((notes[count] ==  '\n') || (notes[count] ==  '\r'))
                notes[count] = '~';
        }
    }

    NotesForm->IsClean(true);
    MainForm->Grid->Refresh();
    MainForm->IsClean(false);

    return;
}

/*-----------------------------------
    Location::GetCentre(int &ctrcol, int &ctrrow)
    Work out the central location coords.
-----------------------------------*/
void Location::GetCentre(int &ctrcol, int &ctrrow)
{
    // Subtract minimums since grid should only display used locations.
    ctrcol = (maxcol + mincol)/2 - mincol;
    ctrrow = (maxrow + minrow)/2 - minrow;
}

/*-----------------------------------
    Location::GetCentre(void)
    Work out the central location coords.
-----------------------------------*/
void Location::ResetCentre(void)
{
    mincol = 64;
    maxcol = -1;
    minrow = 64;
    maxrow = -1;
}

