/*--------------------------------------------------------------------

  World Folio - Offline map viewer/editor for Age of Adventure
  
  Copyright (c) 2001 Phil Dempster <phil@buxton.port5.com>
  http://buxton.port5.com/age/

  This software is based on an original work the copyright in which
  vests in Interactive Broadcasting Ltd (http://www.ibgames.net/).
  Original programming by Alan Lenton with additional programming by
  Stanley Lenton.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Compiled with Borland C++ Builder 3.
    
--------------------------------------------------------------------*/


//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "age_map.h"
#include "location.h"
#include "main.h"
#include "notes.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNotesForm *NotesForm;
//---------------------------------------------------------------------------
__fastcall TNotesForm::TNotesForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void TNotesForm::Status(void)
{
    char buf[32];
    unsigned exits, i;

    //FIXME OnShow...
    NoSpecial->Checked = true;
    NoUser->Checked    = true;
    special = loc->Special();

    switch (special)
    {
    case I_PAWNSHOP:    Pawnshop1->Checked          = true; break;
    case I_CAFEBAR:     Cafebar1->Checked           = true; break;
    case I_RAILWAY:     Railwaystation1->Checked    = true; break;
    case I_PORT:        Port1->Checked              = true; break;
    case I_AIRSHIP:     Airshipterminal1->Checked   = true; break;
//    case I_INSTITUTE:   Institute1->Checked         = true; break;
    case I_HOSPITAL:    Hospital1->Checked          = true; break;
    case I_TELEGRAPH:   Telegraph1->Checked         = true; break;
    case I_RATIONS:     Rations1->Checked           = true; break;
    case I_WEAPON:      Weapons1->Checked           = true; break;
    case I_SKILL:       Training1->Checked          = true; break;
    }
    if (special >= I_USER)
    {
        user    = special;
        special = -1;
    }
    else
    {
        user = -1;
    }

    sprintf(buf, "%d", loc->Row() * 32 + loc->Column());
    FindStatus->Panels->Items[0]->Text = buf;

    sprintf(buf, "%d, %d", loc->Column(), loc->Row());
    FindStatus->Panels->Items[1]->Text = buf;

    exits = loc->Exits();
    *buf = NULL;

    NItem ->Visible = false;
    NEItem->Visible = false;
    EItem ->Visible = false;
    SEItem->Visible = false;
    SItem ->Visible = false;
    SWItem->Visible = false;
    WItem ->Visible = false;
    NWItem->Visible = false;

    for (i=0x01; i<=0x80; i<<=1)
    {
        switch (exits & i)
        {
        case 0x01:  strcat(buf, "N ");  NItem ->Visible=true;   break;
        case 0x02:  strcat(buf, "NE "); NEItem->Visible=true;   break;
        case 0x04:  strcat(buf, "E ");  EItem ->Visible=true;   break;
        case 0x08:  strcat(buf, "SE "); SEItem->Visible=true;   break;
        case 0x10:  strcat(buf, "S ");  SItem ->Visible=true;   break;
        case 0x20:  strcat(buf, "SW "); SWItem->Visible=true;   break;
        case 0x40:  strcat(buf, "W ");  WItem ->Visible=true;   break;
        case 0x80:  strcat(buf, "NW "); NWItem->Visible=true;   break;
        }
    }
    FindStatus->Panels->Items[2]->Text = buf;
}
//---------------------------------------------------------------------------
void __fastcall TNotesForm::ExitItemClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------


void __fastcall TNotesForm::FormClose(TObject *Sender, TCloseAction &Action)
{
    if(!IsClean())
    {
        switch(MessageDlg("Update with changes?",mtConfirmation,TMsgDlgButtons() << mbYes << mbNo << mbCancel,0))
        {
        case mrYes:
            loc->Special(special);
            loc->User(user);
            loc->UpdateNotes();
            IsClean(true);
            Action = caHide;
            break;
        case mrNo:
            IsClean(true);
            Action = caHide;
            break;
        case mrCancel:
            Action = caNone;
            return;
        }
    }
    loc = NULL;
}
//---------------------------------------------------------------------------


void __fastcall TNotesForm::FormResize(TObject *Sender)
{
    int btn_width;

    btn_width = NoteTools->Width / 3;

    DeleteBtn->Width = btn_width;
    ResetBtn->Width  = btn_width;
    SaveBtn->Width   = btn_width;
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::ClearItemClick(TObject *Sender)
{
    NotesMemo->Clear();
    IsClean(false);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::ResetItemClick(TObject *Sender)
{
    NotesMemo->Text = loc->Notes();
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::SaveItemClick(TObject *Sender)
{
    loc->Special(special);
    loc->User(user);
    loc->UpdateNotes();
    IsClean(true);
    Close();
}
//---------------------------------------------------------------------------

// FIXME shouldn't have the  nasty * 2 here then / 2 in age_map.cpp
// in the meantime, use the following fudge routines.
static inline int adj_col(Location *loc)
{
    return (loc->Column() - Location::mincol) * 2;
}

static inline int adj_row(Location *loc)
{
    return (loc->Row() - Location::minrow) * 2;
}


void __fastcall TNotesForm::NItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)+1, adj_row(loc)-1, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::NEItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)+3, adj_row(loc)-1, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::EItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)+3, adj_row(loc)+1, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::SEItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)+3, adj_row(loc)+3, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::SItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)+1, adj_row(loc)+3, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::SWItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)-1, adj_row(loc)+3, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::WItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)-1, adj_row(loc)+1, Left, Top);
}
//---------------------------------------------------------------------------

void __fastcall TNotesForm::NWItemClick(TObject *Sender)
{
    MainForm->ShowNotes(adj_col(loc)-1, adj_row(loc)-1, Left, Top);
}
//---------------------------------------------------------------------------
void __fastcall TNotesForm::SpecialClick(TObject *Sender)
{
    TMenuItem *item = (TMenuItem*)Sender;

    item->Checked = true;
    special = item->Tag;
    special = (special==-1) ? special : special+I_SPECIAL;
    IsClean(false);
}
//---------------------------------------------------------------------------
void __fastcall TNotesForm::UserClick(TObject *Sender)
{
    TMenuItem *item = (TMenuItem*)Sender;

    item->Checked = true;
    user = item->Tag;
    user = (user==-1) ? user : user+I_USER;
    IsClean(false);
}
//---------------------------------------------------------------------------

